@@ -1,9 +1,6 @@
 /* Define if you have GCC.  */
 #undef HAVE_GCC
 
-/* Define if your system is HP-UX.  */
-#undef HAVE_HPUX
-
 /* Define if you have a termcap library.  */
 #undef HAVE_LIBTERMCAP
 
@@ -13,6 +10,9 @@
 /* Define if your system is Linux.  */
 #undef HAVE_LINUX
 
+/* Define if you have a dumb C compiler, like the one running on B.O.S.  */
+#undef HAVE_DUMB_CC
+
 /* Define if you have POSIX compatible terminal interface.  */
 #undef HAVE_POSIX_TTY
 
