@@ -1,6 +1,5 @@
-/* ansi_stdlib.h -- An ANSI Standard stdlib.h. */
-/* A minimal stdlib.h containing extern declarations for those functions
-   that bash uses. */
+/* ansi_stdlib.h -- An ANSI Standard stdlib.h.  A minimal stdlib.h
+   containing extern declarations for those functions that bash uses. */
 
 /* Copyright (C) 1993 Free Software Foundation, Inc.
 
