@@ -1,5 +1,6 @@
-/* fsusage.c -- return space usage of mounted filesystems
-   Copyright (C) 1991, 1992 Free Software Foundation, Inc.
+/* fsusage.c -- Return space usage of mounted filesystems.  */
+
+/* Copyright (C) 1991, 1992 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -61,20 +62,21 @@ int statvfs ();
 
 long
 fs_adjust_blocks (blocks, fromsize, tosize)
-     long blocks;
-     int fromsize, tosize;
+    long blocks;
+    int fromsize, tosize;
 {
-  if (tosize <= 0)
-    abort ();
-  if (fromsize <= 0)
-    return -1;
+    if (tosize <= 0)
+	abort ();
 
-  if (fromsize == tosize)       /* E.g., from 512 to 512.  */
-    return blocks;
-  else if (fromsize > tosize)   /* E.g., from 2048 to 512.  */
-    return blocks * (fromsize / tosize);
-  else                          /* E.g., from 256 to 512.  */
-    return (blocks + (blocks < 0 ? -1 : 1)) / (tosize / fromsize);
+    if (fromsize <= 0)
+	return -1;
+
+    if (fromsize == tosize)       /* E.g., from 512 to 512.  */
+	return blocks;
+    else if (fromsize > tosize)   /* E.g., from 2048 to 512.  */
+	return blocks * (fromsize / tosize);
+    else                          /* E.g., from 256 to 512.  */
+	return (blocks + (blocks < 0 ? -1 : 1)) / (tosize / fromsize);
 }
 
 /* Fill in the fields of FSP with information about space usage for
@@ -83,56 +85,56 @@ fs_adjust_blocks (blocks, fromsize, tosize)
 
 int
 get_fs_usage (path, fsp)
-     char *path;
-     struct fs_usage *fsp;
+    char *path;
+    struct fs_usage *fsp;
 {
 #if defined (STAT_STATFS3_OSF1)
-  struct statfs fsd;
+    struct statfs fsd;
 
-  if (statfs (path, &fsd, sizeof (struct statfs)) != 0)
-    return -1;
-#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_fsize, 512)
+    if (statfs (path, &fsd, sizeof (struct statfs)) != 0)
+	return -1;
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), (int)fsd.f_fsize, 512)
 #endif /* STAT_STATFS3_OSF1 */
 
 #ifdef STAT_STATFS2_FS_DATA     /* Ultrix.  */
-  struct fs_data fsd;
+    struct fs_data fsd;
 
-  if (statfs (path, &fsd) != 1)
-    return -1;
+    if (statfs (path, &fsd) != 1)
+	return -1;
 #define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), 1024, 512)
-  fsp->fsu_blocks = CONVERT_BLOCKS (fsd.fd_req.btot);
-  fsp->fsu_bfree = CONVERT_BLOCKS (fsd.fd_req.bfree);
-  fsp->fsu_bavail = CONVERT_BLOCKS (fsd.fd_req.bfreen);
-  fsp->fsu_files = fsd.fd_req.gtot;
-  fsp->fsu_ffree = fsd.fd_req.gfree;
+    fsp->fsu_blocks = CONVERT_BLOCKS (fsd.fd_req.btot);
+    fsp->fsu_bfree = CONVERT_BLOCKS (fsd.fd_req.bfree);
+    fsp->fsu_bavail = CONVERT_BLOCKS (fsd.fd_req.bfreen);
+    fsp->fsu_files = fsd.fd_req.gtot;
+    fsp->fsu_ffree = fsd.fd_req.gfree;
 #endif
 
 #ifdef STAT_STATFS2_BSIZE       /* 4.3BSD, SunOS 4, HP-UX, AIX.  */
-  struct statfs fsd;
+    struct statfs fsd;
 
-  if (statfs (path, &fsd) < 0)
-    return -1;
-#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_bsize, 512)
+    if (statfs (path, &fsd) < 0)
+	return -1;
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), (int)fsd.f_bsize, 512)
 #endif
 
 #ifdef STAT_STATFS2_FSIZE       /* 4.4BSD.  */
-  struct statfs fsd;
+    struct statfs fsd;
 
-  if (statfs (path, &fsd) < 0)
-    return -1;
-#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_fsize, 512)
+    if (statfs (path, &fsd) < 0)
+	return -1;
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), (int)fsd.f_fsize, 512)
 #endif
 
 #ifdef STAT_STATFS4             /* SVR3, Dynix, Irix, AIX.  */
-  struct statfs fsd;
+    struct statfs fsd;
 
-  if (statfs (path, &fsd, sizeof fsd, 0) < 0)
-    return -1;
-  /* Empirically, the block counts on most SVR3 and SVR3-derived
-     systems seem to always be in terms of 512-byte blocks,
-     no matter what value f_bsize has.  */
+    if (statfs (path, &fsd, sizeof fsd, 0) < 0)
+	return -1;
+    /* Empirically, the block counts on most SVR3 and SVR3-derived
+       systems seem to always be in terms of 512-byte blocks,
+       no matter what value f_bsize has.  */
 # if _AIX
-#  define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_bsize, 512)
+#  define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), (int)fsd.f_bsize, 512)
 # else
 #  define CONVERT_BLOCKS(b) (b)
 #  ifndef _SEQUENT_             /* _SEQUENT_ is DYNIX/ptx.  */
@@ -144,24 +146,25 @@ get_fs_usage (path, fsp)
 #endif
 
 #ifdef STAT_STATVFS             /* SVR4.  */
-  struct statvfs fsd;
+    struct statvfs fsd;
 
-  if (statvfs (path, &fsd) < 0)
-    return -1;
-  /* f_frsize isn't guaranteed to be supported.  */
+    if (statvfs (path, &fsd) < 0)
+	return -1;
+
+    /* f_frsize isn't guaranteed to be supported.  */
 #define CONVERT_BLOCKS(b) \
-  fs_adjust_blocks ((b), fsd.f_frsize ? fsd.f_frsize : fsd.f_bsize, 512)
+    fs_adjust_blocks((b), fsd.f_frsize?(int)fsd.f_frsize:(int)fsd.f_bsize, 512)
 #endif
 
 #if defined(CONVERT_BLOCKS) && !defined(STAT_STATFS2_FS_DATA) && !defined(STAT_READ_FILSYS) /* !Ultrix && !SVR2.  */
-  fsp->fsu_blocks = CONVERT_BLOCKS (fsd.f_blocks);
-  fsp->fsu_bfree = CONVERT_BLOCKS (fsd.f_bfree);
-  fsp->fsu_bavail = CONVERT_BLOCKS (fsd.f_bavail);
-  fsp->fsu_files = fsd.f_files;
-  fsp->fsu_ffree = fsd.f_ffree;
+    fsp->fsu_blocks = CONVERT_BLOCKS (fsd.f_blocks);
+    fsp->fsu_bfree = CONVERT_BLOCKS (fsd.f_bfree);
+    fsp->fsu_bavail = CONVERT_BLOCKS (fsd.f_bavail);
+    fsp->fsu_files = fsd.f_files;
+    fsp->fsu_ffree = fsd.f_ffree;
 #endif
 
-  return 0;
+    return 0;
 }
 
 #if defined(_AIX) && defined(_I386)
@@ -169,25 +172,28 @@ get_fs_usage (path, fsp)
 
 int
 statfs (path, fsb)
-     char *path;
-     struct statfs *fsb;
+    char *path;
+    struct statfs *fsb;
 {
-  struct stat stats;
-  struct dustat fsd;
+    struct stat stats;
+    struct dustat fsd;
 
-  if (xstat (path, &stats))
-    return -1;
-  if (dustat (stats.st_dev, 0, &fsd, sizeof (fsd)))
-    return -1;
-  fsb->f_type   = 0;
-  fsb->f_bsize  = fsd.du_bsize;
-  fsb->f_blocks = fsd.du_fsize - fsd.du_isize;
-  fsb->f_bfree  = fsd.du_tfree;
-  fsb->f_bavail = fsd.du_tfree;
-  fsb->f_files  = (fsd.du_isize - 2) * fsd.du_inopb;
-  fsb->f_ffree  = fsd.du_tinode;
-  fsb->f_fsid.val[0] = fsd.du_site;
-  fsb->f_fsid.val[1] = fsd.du_pckno;
-  return 0;
+    if (xstat(path, &stats))
+	return -1;
+
+    if (dustat(stats.st_dev, 0, &fsd, sizeof (fsd)))
+	return -1;
+
+    fsb->f_type   = 0;
+    fsb->f_bsize  = fsd.du_bsize;
+    fsb->f_blocks = fsd.du_fsize - fsd.du_isize;
+    fsb->f_bfree  = fsd.du_tfree;
+    fsb->f_bavail = fsd.du_tfree;
+    fsb->f_files  = (fsd.du_isize - 2) * fsd.du_inopb;
+    fsb->f_ffree  = fsd.du_tinode;
+    fsb->f_fsid.val[0] = fsd.du_site;
+    fsb->f_fsid.val[1] = fsd.du_pckno;
+
+    return 0;
 }
 #endif /* _AIX && _I386 */
