@@ -1,4 +1,4 @@
-/* gitps.c -- a process viewer/killer utility. */
+/* gitps.c -- A process viewer/killer utility.  */
 
 /* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
 
@@ -32,6 +32,11 @@
 #endif /* !HAVE_STDLIB_H */
 
 #include <sys/types.h>
+
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
 #include <ctype.h>
 #include <limits.h>
 #include "file.h"
@@ -134,8 +139,6 @@ int  signal_type = 11;  /* index of SIGTERM in signals table */
 pid_t pid;
 char *home;
 char *program;
-char *tty_name;
-size_t tty_name_len;
 char *header_text;
 char *TempDirectory;
 int UseLastScreenChar;
@@ -150,7 +153,7 @@ window_t *title_win, *header_win, *screen_win, *status_win;
 #ifdef HAVE_GCC
 static char title_text[] = " "PRODUCT" "VERSION" - Process Viewer/Killer";
 #else
-static char title_text[] = " GNU Interactive Tools 4.3.9 - Process Viewer/Killer";
+static char title_text[] = " GNU Interactive Tools 4.3.16 - Process Viewer/Killer";
 #endif /* !HAVE_GCC */
 static char *GitPsModeHelp;
 static char no_perm[] = "not owner !";
@@ -573,7 +576,7 @@ read_keys(keys)
 {
     char *contents;
     char key_seq[80];
-    int i, j, need_convertion;
+    int i, j, need_conversion;
 
 
     for (i = keys; i < MAX_KEYS; i++)
@@ -587,11 +590,29 @@ read_keys(keys)
 	{
 	    char *key_seq_ptr = tty_get_symbol_key_seq(key_seq);
 
-	    if (!(need_convertion = key_seq_ptr == NULL))
+	    if (key_seq_ptr)
+	    {
+		/* Ignore empty/invalid key sequences.  */
+		if (*key_seq_ptr == '\0')
+		    continue;
+
+		/* We got the key sequence in the correct form, as
+		   returned by tgetstr, so there is no need for
+		   further conversion.  */
 		strcpy(key_seq, key_seq_ptr);
+		need_conversion = 0;
+	    }
+	    else
+	    {
+		/* This is not a TERMCAP symbol, it is a key sequence
+		   that we will have to convert it with
+		   tty_key_convert() into a machine usable form before
+		   using it.  */
+		need_conversion = 1;
+	    }
 	}
 	else
-	    need_convertion = 1;
+	    need_conversion = 1;
 
 	if (contents == NULL)
 	    continue;
@@ -602,8 +623,8 @@ read_keys(keys)
 
 	if (j < BUILTIN_OPERATIONS)
 	{
-	    if (!need_convertion || tty_key_convert((unsigned char *)key_seq))
-		tty_key_list_insert((unsigned char *)key_seq, (void *)j);
+	    if (!need_conversion || tty_key_convert((unsigned char *)key_seq))
+		tty_key_list_insert((unsigned char *)key_seq, built_in[j]);
 	}
 	else
 	    fprintf(stderr, "%s: invalid built-in operation: %s.\n",
@@ -759,7 +780,7 @@ restart:
     fclose(stdout_log);
 
     settitle();
-    setstatus(NULL);
+    setstatus((char *)NULL);
     setsignal();
     setheader();
 
@@ -770,7 +791,7 @@ restart:
     while (1)
     {
 	ks  = tty_get_key(&repeat_count);
-	key = (int)ks->aux_data;
+	key = ((char *)ks->aux_data - (char *)built_in) / MAX_BUILTIN_NAME;
 
 	switch (key)
 	{
@@ -964,7 +985,7 @@ restart:
 		    xfree(tmp);
 		    errno = 0;
 		    tty_get_key(NULL);
-		    setstatus(NULL);
+		    setstatus((char *)NULL);
 		    setsignal();
 		}
 		break;
