@@ -1,4 +1,4 @@
-/* History.c -- standalone history library */
+/* history.c -- Standalone history library.  */
 
 /* Copyright (C) 1989, 1992 Free Software Foundation, Inc.
 
@@ -225,8 +225,8 @@ add_history (string)
       /* If there is something in the slot, then remove it. */
       if (the_history[0])
 	{
-	  free (the_history[0]->line);
-	  free (the_history[0]);
+	  xfree (the_history[0]->line);
+	  xfree (the_history[0]);
 	}
 
       /* Copy the rest of the entries, moving down one slot. */
@@ -457,8 +457,8 @@ stifle_history (max)
       /* This loses because we cannot free the data. */
       for (i = 0; i < (history_length - max); i++)
 	{
-	  free (the_history[i]->line);
-	  free (the_history[i]);
+	  xfree (the_history[i]->line);
+	  xfree (the_history[i]);
 	}
 
       history_base = i;
@@ -560,10 +560,10 @@ read_history_range (filename, from, to)
 	close (file);
 
       if (input)
-	free (input);
+	xfree (input);
 
       if (buffer)
-	free (buffer);
+	xfree (buffer);
 
       return (errno);
     }
@@ -608,10 +608,10 @@ read_history_range (filename, from, to)
       }
 
   if (input)
-    free (input);
+    xfree (input);
 
   if (buffer)
-    free (buffer);
+    xfree (buffer);
 
   return (0);
 }
@@ -671,9 +671,9 @@ history_truncate_file (fname, lines)
 
  truncate_exit:
   if (buffer)
-    free (buffer);
+    xfree (buffer);
 
-  free (filename);
+  xfree (filename);
   return 0;
 }
 
@@ -697,7 +697,7 @@ history_do_write (filename, nelements, overwrite)
   if ((file = open (output, mode, 0666)) == -1)
     {
       if (output)
-	free (output);
+	xfree (output);
 
       return (errno);
     }
@@ -727,13 +727,13 @@ history_do_write (filename, nelements, overwrite)
       }
 
     write (file, buffer, buffer_size);
-    free (buffer);
+    xfree (buffer);
   }
 
   close (file);
 
   if (output)
-    free (output);
+    xfree (output);
 
   return (0);
 }
@@ -965,7 +965,7 @@ get_history_event (string, caller_index, delimiting_quote)
   *caller_index = i;
 
 #define FAIL_SEARCH() \
-  do { history_offset = history_length; free (temp) ; return (char *)NULL; } while (0)
+  do { history_offset = history_length; xfree (temp) ; return (char *)NULL; } while (0)
 
   search_mode = substring_okay ? NON_ANCHORED_SEARCH : ANCHORED_SEARCH;
   while (1)
@@ -985,11 +985,11 @@ get_history_event (string, caller_index, delimiting_quote)
 	  if (substring_okay)
 	    {
 	      if (search_string)
-		free (search_string);
+		xfree (search_string);
 	      search_string = temp;
 	    }
 	  else
-	    free (temp);
+	    xfree (temp);
 	  return (entry->line);
 	}
 
@@ -1183,7 +1183,7 @@ postproc_subst_rhs ()
 	}
     }
   new[j] = '\0';
-  free (subst_rhs);
+  xfree (subst_rhs);
   subst_rhs = new;
   subst_rhs_len = j;
 }
@@ -1242,7 +1242,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
   if (!event)
     {
       *ret_string = hist_error (string, start, i, EVENT_NOT_FOUND);
-      free (result);
+      xfree (result);
       return (-1);
     }
 
@@ -1256,7 +1256,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
   if (word_spec == (char *)&error_pointer)
     {
       *ret_string = hist_error (string, starting_index, i, BAD_WORD_SPEC);
-      free (result);
+      xfree (result);
       return (-1);
     }
 
@@ -1266,7 +1266,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
   else
     {
       temp = savestring (word_spec);
-      free (word_spec);
+      xfree (word_spec);
     }
 
   /* Perhaps there are other modifiers involved.  Do what they say. */
@@ -1288,8 +1288,8 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	{
 	default:
 	  *ret_string = hist_error (string, i+1, i+2, BAD_MODIFIER);
-	  free (result);
-	  free (temp);
+	  xfree (result);
+	  xfree (temp);
 	  return -1;
 
 #if defined (SHELL)
@@ -1316,7 +1316,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	    {
 	      tstr++;
 	      t = savestring (tstr);
-	      free (temp);
+	      xfree (temp);
 	      temp = t;
 	    }
 	  break;
@@ -1341,7 +1341,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	  if (tstr)
 	    {
 	      t = savestring (tstr);
-	      free (temp);
+	      xfree (temp);
 	      temp = t;
 	    }
 	  break;
@@ -1373,7 +1373,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 		if (t)
 		  {
 		    if (subst_lhs)
-		      free (subst_lhs);
+		      xfree (subst_lhs);
 		    subst_lhs = t;
 		  }
 		else if (!subst_lhs)
@@ -1394,13 +1394,13 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 		if (subst_lhs_len == 0)
 		  {
 		    *ret_string = hist_error (string, starting_index, i, SUBST_FAILED);
-		    free (result);
-		    free (temp);
+		    xfree (result);
+		    xfree (temp);
 		    return -1;
 		  }
 
 		if (subst_rhs)
-		  free (subst_rhs);
+		  xfree (subst_rhs);
 		subst_rhs = get_subst_pattern (string, &i, delimiter, 1, &subst_rhs_len);
 
 		/* If `&' appears in the rhs, it's supposed to be replaced
@@ -1416,8 +1416,8 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	    if (subst_lhs_len > l_temp)
 	      {
 		*ret_string = hist_error (string, starting_index, i, SUBST_FAILED);
-		free (result);
-		free (temp);
+		xfree (result);
+		xfree (temp);
 		return (-1);
 	      }
 
@@ -1434,7 +1434,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 			   temp + si + subst_lhs_len,
 			   l_temp - (si + subst_lhs_len));
 		  new_event[len] = '\0';
-		  free (temp);
+		  xfree (temp);
 		  temp = new_event;
 
 		  failed = 0;
@@ -1460,8 +1460,8 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	      continue;         /* don't want to increment i */
 
 	    *ret_string = hist_error (string, starting_index, i, SUBST_FAILED);
-	    free (result);
-	    free (temp);
+	    xfree (result);
+	    xfree (temp);
 	    return (-1);
 	  }
 	}
@@ -1483,7 +1483,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
       else
 	x = savestring (temp);
 
-      free (temp);
+      xfree (temp);
       temp = x;
     }
 #endif /* SHELL */
@@ -1492,7 +1492,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
   if (n > result_len)
     result = xrealloc (result, n + 1);
   strcpy (result, temp);
-  free (temp);
+  xfree (temp);
 
   *end_index_ptr = i;
   *ret_string = result;
@@ -1628,7 +1628,7 @@ history_expand (hstring, output)
 
       if (string[i] != history_expansion_char)
 	{
-	  free (result);
+	  xfree (result);
 	  *output = savestring (string);
 	  return (0);
 	}
@@ -1674,7 +1674,7 @@ history_expand (hstring, output)
 	    strncpy (temp, string + quote, slen);
 	    temp[slen - 1] = '\0';
 	    ADD_STRING (temp);
-	    free (temp);
+	    xfree (temp);
 	    break;
 	  }
 #endif /* SHELL */
@@ -1704,7 +1704,7 @@ history_expand (hstring, output)
 		  temp = xmalloc (1 + strlen (result));
 		  strcpy (temp, result);
 		  ADD_STRING (temp);
-		  free (temp);
+		  xfree (temp);
 		}
 	      i++;
 	      break;
@@ -1715,9 +1715,9 @@ history_expand (hstring, output)
 	  if (r < 0)
 	    {
 	      *output = temp;
-	      free (result);
+	      xfree (result);
 	      if (string != hstring)
-		free (string);
+		xfree (string);
 	      return -1;
 	    }
 	  else
@@ -1727,7 +1727,7 @@ history_expand (hstring, output)
 		  modified++;
 		  if (*temp)
 		    ADD_STRING (temp);
-		  free (temp);
+		  xfree (temp);
 		}
 	      only_printing = r == 1;
 	      i = eindex;
@@ -1738,7 +1738,7 @@ history_expand (hstring, output)
 
   *output = result;
   if (string != hstring)
-    free (string);
+    xfree (string);
 
   if (only_printing)
     {
@@ -1906,8 +1906,8 @@ history_arg_extract (first, last, string)
     }
 
   for (i = 0; i < len; i++)
-    free (list[i]);
-  free (list);
+    xfree (list[i]);
+  xfree (list);
 
   return (result);
 }
