@@ -1,6 +1,6 @@
-/* inputline.c -- input line management functions. */
+/* inputline.c -- Input line management functions.  */
 
-/* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1993, 1994, 1995, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,12 +25,20 @@
 
 #include <stdio.h>
 
+#include <sys/types.h>
+
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #else /* !HAVE_STDLIB_H */
 #include "ansi_stdlib.h"
 #endif /* !HAVE_STDLIB_H */
 
+#include <ctype.h>
+
 #include "xstring.h"
 #include "xmalloc.h"
 #include "configure.h"
@@ -81,30 +89,34 @@ static int InputLineColors[INPUTLINE_FIELDS] =
 
 
 /* Basic input line operations.  */
-#define IL_NO_OPERATION                          0
-#define IL_BACKWARD_CHAR                         1
-#define IL_FORWARD_CHAR                          2
-#define IL_BACKWARD_WORD                         3
-#define IL_FORWARD_WORD                          4
-#define IL_BEGINNING_OF_LINE                     5
-#define IL_END_OF_LINE                           6
-#define IL_INSERT_CHAR                           7
-#define IL_DELETE_CHAR                           8
-#define IL_BACKWARD_DELETE_CHAR                  9
-#define IL_BACKWARD_KILL_WORD                   10
-#define IL_RESET_LINE                           11
-#define IL_KILL_LINE                            12
-#define IL_KILL_TO_BEGINNING_OF_LINE            13
-#define IL_KILL_TO_END_OF_LINE                  14
-#define IL_JUST_ONE_SPACE                       15
-#define IL_DELETE_HORIZONTAL_SPACE              16
-#define IL_SET_STATIC_TEXT                      17
-#define IL_INSERT_TEXT                          18
-#define IL_SET_MARK                             19
-#define IL_KILL_REGION                          20
-#define IL_KILL_RING_SAVE                       21
-#define IL_YANK                                 22
-#define IL_EXCHANGE_POINT_AND_MARK              23
+#define IL_NO_OPERATION				 0
+#define IL_BACKWARD_CHAR			 1
+#define IL_FORWARD_CHAR				 2
+#define IL_BACKWARD_WORD			 3
+#define IL_FORWARD_WORD				 4
+#define IL_BEGINNING_OF_LINE			 5
+#define IL_END_OF_LINE				 6
+#define IL_INSERT_CHAR				 7
+#define IL_DELETE_CHAR				 8
+#define IL_BACKWARD_DELETE_CHAR			 9
+#define IL_KILL_WORD				10
+#define IL_BACKWARD_KILL_WORD			11
+#define IL_RESET_LINE				12
+#define IL_KILL_LINE				13
+#define IL_KILL_TO_BEGINNING_OF_LINE		14
+#define IL_KILL_TO_END_OF_LINE			15
+#define IL_JUST_ONE_SPACE			16
+#define IL_DELETE_HORIZONTAL_SPACE		17
+#define IL_DOWNCASE_WORD			18
+#define IL_UPCASE_WORD				19
+#define IL_CAPITALIZE_WORD			20
+#define IL_SET_STATIC_TEXT			21
+#define IL_INSERT_TEXT				22
+#define IL_SET_MARK				23
+#define IL_KILL_REGION				24
+#define IL_KILL_RING_SAVE			25
+#define IL_YANK					26
+#define IL_EXCHANGE_POINT_AND_MARK		27
 
 
 #define IL_HORIZONTAL_SCROLL                    20
@@ -138,8 +150,10 @@ il_separator(c)
 }
 
 
-/* Store the text between the mark and the point into the kill_ring if
-   flags & IL_STORE.  Kill it if flags & IL_KILL.  */
+/*
+ * Store the text between the mark and the point into the kill_ring if
+ * flags & IL_STORE.  Kill it if flags & IL_KILL.
+ */
 
 static void
 il_region_command(flags)
@@ -190,7 +204,9 @@ il_region_command(flags)
 }
 
 
-/* Free an input line structure.  */
+/*
+ * Free an input line structure.
+ */
 
 void
 il_free(some_il)
@@ -209,7 +225,9 @@ il_free(some_il)
 }
 
 
-/* The input_line constructor.  */
+/*
+ * The input_line constructor.
+ */
 
 void
 il_init(_columns, _begin_y)
@@ -249,7 +267,9 @@ il_init(_columns, _begin_y)
 }
 
 
-/* Input line destructor.  */
+/*
+ * Input line destructor.
+ */
 
 void
 il_end()
@@ -262,8 +282,10 @@ il_end()
 }
 
 
-/* Save the entire input_line structure.  Returns a pointer to the saved
-   structure.  */
+/*
+ * Save the entire input_line structure.  Returns a pointer to the saved
+ * structure.
+ */
 
 input_line_t *
 il_save()
@@ -285,7 +307,9 @@ il_save()
 }
 
 
-/* Restore the input_line.  */
+/*
+ * Restore the input_line.
+ */
 
 void
 il_restore(saved_il)
@@ -299,7 +323,10 @@ il_restore(saved_il)
 }
 
 
-/* Set the echo state.  Returns the previous state of the echo flag.  */
+/*
+ * Set the echo state.  Returns the previous state of the echo flag.
+ */
+
 int
 il_echo(echo)
     int echo;
@@ -311,7 +338,9 @@ il_echo(echo)
 }
 
 
-/* Check if the input line is empty.  */
+/*
+ * Check if the input line is empty.
+ */
 
 int
 il_is_empty()
@@ -320,7 +349,9 @@ il_is_empty()
 }
 
 
-/* Set mark where point is.  */
+/*
+ * Set mark where point is.
+ */
 
 void
 il_set_mark()
@@ -329,8 +360,10 @@ il_set_mark()
 }
 
 
-/* Kill between point and mark.  The text is deleted but saved in the
-   kill ring.  */
+/*
+ * Kill between point and mark.  The text is deleted but saved in the
+ * kill ring.
+ */
 
 void
 il_kill_region()
@@ -340,7 +373,9 @@ il_kill_region()
 }
 
 
-/* Save the region as if killed, but don't kill it.  */
+/*
+ * Save the region as if killed, but don't kill it.
+ */
 
 void
 il_kill_ring_save()
@@ -350,7 +385,9 @@ il_kill_ring_save()
 }
 
 
-/* Reinsert the last stretch of killed text.  */
+/*
+ * Reinsert the last stretch of killed text.
+ */
 
 void
 il_yank()
@@ -363,7 +400,9 @@ il_yank()
 }
 
 
-/* Exchange point and mark.  */
+/*
+ * Exchange point and mark.
+ */
 
 void
 il_exchange_point_and_mark()
@@ -374,7 +413,9 @@ il_exchange_point_and_mark()
 }
 
 
-/* Move the point backward one character.  */
+/*
+ * Move the point backward one character.
+ */
 
 void
 il_backward_char()
@@ -387,7 +428,9 @@ il_backward_char()
 }
 
 
-/* Move the point forward one character.  */
+/*
+ * Move the point forward one character.
+ */
 
 void
 il_forward_char()
@@ -400,7 +443,9 @@ il_forward_char()
 }
 
 
-/* Move the point backward one word.  */
+/*
+ * Move the point backward one word.
+ */
 
 void
 il_backward_word()
@@ -420,7 +465,9 @@ il_backward_word()
 }
 
 
-/* Move the point forward one word.  */
+/*
+ * Move the point forward one word.
+ */
 
 void
 il_forward_word()
@@ -440,7 +487,9 @@ il_forward_word()
 }
 
 
-/* Move the point at the beginning of the line.  */
+/*
+ * Move the point at the beginning of the line.
+ */
 
 void
 il_beginning_of_line()
@@ -451,7 +500,9 @@ il_beginning_of_line()
 }
 
 
-/* Move the point at the end of the line.  */
+/*
+ * Move the point at the end of the line.
+ */
 
 void
 il_end_of_line()
@@ -462,7 +513,9 @@ il_end_of_line()
 }
 
 
-/* Insert 'c' at the point position.  */
+/*
+ * Insert 'c' at the point position.
+ */
 
 void
 il_insert_char(c)
@@ -486,7 +539,9 @@ il_insert_char(c)
 }
 
 
-/* Delete the character at the point.  */
+/*
+ * Delete the character at the point.
+ */
 
 void
 il_delete_char()
@@ -508,7 +563,9 @@ il_delete_char()
 }
 
 
-/* Delete the character before the point.  */
+/*
+ * Delete the character before the point.
+ */
 
 void
 il_backward_delete_char()
@@ -531,7 +588,38 @@ il_backward_delete_char()
 }
 
 
-/* Kill characters backward until encountering the end of a word.  */
+/*
+ * Kill characters forward until encountering the end of a word.
+ */
+
+void
+il_kill_word()
+{
+    int end_of_word;
+    int old_mark = il->mark;
+
+    il_set_mark();
+    il_forward_word();
+
+    end_of_word = il->point;
+
+    il_region_command(IL_STORE | IL_KILL);
+
+    if (old_mark <= il->point)
+	il->mark = old_mark;
+    else
+	if (old_mark >= end_of_word)
+	    il->mark = old_mark - (end_of_word - il->point);
+	else
+	    il->mark = il->point;
+
+    il->last_operation = IL_KILL_WORD;
+}
+
+
+/*
+ * Kill characters backward until encountering the end of a word.
+ */
 
 void
 il_backward_kill_word()
@@ -555,7 +643,9 @@ il_backward_kill_word()
 }
 
 
-/* Delete the entire line (both the static & dynamic parts).  */
+/*
+ * Delete the entire line (both the static & dynamic parts).
+ */
 
 void
 il_reset_line()
@@ -574,7 +664,9 @@ il_reset_line()
 }
 
 
-/* Delete the entire dynamic part of the line.  */
+/*
+ * Delete the entire dynamic part of the line.
+ */
 
 void
 il_kill_line(store)
@@ -590,7 +682,9 @@ il_kill_line(store)
 }
 
 
-/* Delete the text between the beginning of the line and the point.  */
+/*
+ * Delete the text between the beginning of the line and the point.
+ */
 
 void
 il_kill_to_beginning_of_line()
@@ -607,7 +701,9 @@ il_kill_to_beginning_of_line()
 }
 
 
-/* Delete the text between the point and the end of line.  */
+/*
+ * Delete the text between the point and the end of line.
+ */
 
 void
 il_kill_to_end_of_line()
@@ -623,8 +719,10 @@ il_kill_to_end_of_line()
 }
 
 
-/* Remove all the tabs and spaces arround the point, leaving just one
-   space.  */
+/*
+ * Remove all the tabs and spaces arround the point, leaving just one
+ * space.
+ */
 
 void
 il_just_one_space()
@@ -638,7 +736,9 @@ il_just_one_space()
 }
 
 
-/* Remove all the tabs and spaces arround the point.  */
+/*
+ * Remove all the tabs and spaces arround the point.
+ */
 
 void
 il_delete_horizontal_space()
@@ -656,8 +756,86 @@ il_delete_horizontal_space()
 }
 
 
-/* Set up the static text at the beginning of the input line.  This text cannot
-   be modified by normal editing commands.  */
+/*
+ * Convert the following word to lower case, moving over.
+ */
+
+void
+il_downcase_word()
+{
+    if (il->point < il->length)
+    {
+	int i, previous_point = il->point;
+
+	/* Move to the end of the word.  */
+	il_forward_word();
+
+	for (i = previous_point; i < il->point; i++)
+	    il->buffer[i] = tolower(il->buffer[i]);
+
+	il->last_operation = IL_DOWNCASE_WORD;
+    }
+}
+
+
+/*
+ * Convert the following word to upper case, moving over.
+ */
+
+void
+il_upcase_word()
+{
+    if (il->point < il->length)
+    {
+	int i, previous_point = il->point;
+
+	/* Move to the end of the word.  */
+	il_forward_word();
+
+	for (i = previous_point; i < il->point; i++)
+	    il->buffer[i] = toupper(il->buffer[i]);
+
+	il->last_operation = IL_UPCASE_WORD;
+    }
+}
+
+
+/*
+ * Capitalize the following word, moving over.  This gives the word
+ * a first character in upper case and the rest lower case.
+ */
+
+void
+il_capitalize_word()
+{
+    if (il->point < il->length)
+    {
+	int i, first = 1, previous_point = il->point;
+
+	/* Move to the end of the word.  */
+	il_forward_word();
+
+	for (i = previous_point; i < il->point; i++)
+	    if (isalnum(il->buffer[i]))
+	    {
+		if (first)
+		{
+		    il->buffer[i] = toupper(il->buffer[i]);
+		    first = 0;
+		}
+		else
+		    il->buffer[i] = tolower(il->buffer[i]);
+	    }
+
+	il->last_operation = IL_CAPITALIZE_WORD;
+    }
+}
+
+
+/*
+ * Set up the static text at the beginning of the input line.  This
+ * text cannot be modified by normal editing commands.
+ */
 
 void
 il_set_static_text(static_text)
@@ -688,7 +866,9 @@ il_set_static_text(static_text)
 }
 
 
-/* Insert 'text' at the point position.  */
+/*
+ * Insert 'text' at the point position.
+ */
 
 void
 il_insert_text(text)
@@ -718,7 +898,9 @@ il_insert_text(text)
 }
 
 
-/* Update the point.  */
+/*
+ * Update the point.
+ */
 
 void
 il_update_point()
@@ -737,7 +919,9 @@ il_update_point()
 }
 
 
-/* Update the entire input line.  */
+/*
+ * Update the entire input line.
+ */
 
 void
 il_update()
@@ -785,7 +969,9 @@ il_update()
 }
 
 
-/* Update the entire input line and the point.  */
+/*
+ * Update the entire input line and the point.
+ */
 
 void
 il_full_update()
@@ -795,9 +981,11 @@ il_full_update()
 }
 
 
-/* Get the input line contents (only the dynamic part).
-   dest is a pointer to a NULL pointer or a pointer to a pointer allocated
-   with xmalloc().  */
+/*
+ * Get the input line contents (only the dynamic part).
+ * dest is a pointer to a NULL pointer or a pointer to a pointer allocated
+ * with xmalloc().
+ */
 
 int
 il_get_contents(dest)
@@ -809,7 +997,9 @@ il_get_contents(dest)
 }
 
 
-/* Display a message into the input line.  */
+/*
+ * Display a message into the input line.
+ */
 
 void
 il_message(message)
@@ -829,7 +1019,9 @@ il_set_error_flag(flag)
 }
 
 
-/* input_line interface to the history library.  */
+/*
+ * inputline's interface to the history library.
+ */
 
 void
 il_history(dir)
