@@ -1,6 +1,6 @@
-/* inputline.h -- prototypes for the functions in inputline.c. */
+/* inputline.h -- Prototypes for the functions in inputline.c.  */
 
-/* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1993, 1994, 1995, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,6 +25,10 @@
 
 #include <sys/types.h>
 
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
 #include "stdc.h"
 #include "window.h"
 
@@ -41,12 +45,13 @@
 
 
 /* Some #defines used in external input line interfaces.  */
-#define IL_FREEZED       0
-#define IL_EDIT          1
-#define IL_MOVE          2
-#define IL_BEEP          4
-#define IL_ERROR         8
-#define IL_SAVE         16
+#define IL_FREEZED	 0
+#define IL_EDIT		 1
+#define IL_MOVE		 2
+#define IL_BEEP		 4
+#define IL_ERROR	 8
+#define IL_SAVE		16
+#define IL_HOME		32
 
 #define IL_ISEARCH_ACTION_FAILED       -1
 #define IL_ISEARCH_ACTION_NONE          0
@@ -78,43 +83,47 @@ typedef struct
 
 extern input_line_t *il;
 
-extern void il_free			 PROTO ((input_line_t *));
-extern void il_init			 PROTO ((size_t, size_t));
-extern void il_end			 PROTO (());
-extern input_line_t *il_save		 PROTO (());
-extern void il_restore			 PROTO ((input_line_t *));
-extern int  il_echo			 PROTO ((int));
-extern int  il_is_empty 		 PROTO (());
-extern void il_set_mark 		 PROTO (());
-extern void il_kill_region 		 PROTO (());
-extern void il_kill_ring_save 		 PROTO (());
-extern void il_yank 			 PROTO (());
-extern void il_exchange_point_and_mark	 PROTO (());
-extern void il_backward_char		 PROTO (());
-extern void il_forward_char		 PROTO (());
-extern void il_backward_word		 PROTO (());
-extern void il_forward_word		 PROTO (());
-extern void il_beginning_of_line	 PROTO (());
-extern void il_end_of_line		 PROTO (());
-extern void il_insert_char		 PROTO ((char));
-extern void il_delete_char		 PROTO (());
-extern void il_backward_delete_char	 PROTO (());
-extern void il_backward_kill_word	 PROTO (());
-extern void il_reset_line		 PROTO (());
-extern void il_kill_line		 PROTO ((int));
+extern void il_free PROTO ((input_line_t *));
+extern void il_init PROTO ((size_t, size_t));
+extern void il_end PROTO (());
+extern input_line_t *il_save PROTO (());
+extern void il_restore PROTO ((input_line_t *));
+extern int il_echo PROTO ((int));
+extern int il_is_empty PROTO (());
+extern void il_set_mark PROTO (());
+extern void il_kill_region PROTO (());
+extern void il_kill_ring_save PROTO (());
+extern void il_yank PROTO (());
+extern void il_exchange_point_and_mark PROTO (());
+extern void il_backward_char PROTO (());
+extern void il_forward_char PROTO (());
+extern void il_backward_word PROTO (());
+extern void il_forward_word PROTO (());
+extern void il_beginning_of_line PROTO (());
+extern void il_end_of_line PROTO (());
+extern void il_insert_char PROTO ((char));
+extern void il_delete_char PROTO (());
+extern void il_backward_delete_char PROTO (());
+extern void il_kill_word PROTO (());
+extern void il_backward_kill_word PROTO (());
+extern void il_reset_line PROTO (());
+extern void il_kill_line PROTO ((int));
 extern void il_kill_to_beginning_of_line PROTO (());
-extern void il_kill_to_end_of_line	 PROTO (());
-extern void il_just_one_space		 PROTO (());
-extern void il_delete_horizontal_space	 PROTO (());
-extern void il_set_static_text		 PROTO ((char *));
-extern void il_insert_text		 PROTO ((char *));
-extern void il_update_point		 PROTO (());
-extern void il_update			 PROTO (());
-extern void il_full_update		 PROTO (());
-extern int  il_get_contents		 PROTO ((char **));
-extern void il_message			 PROTO ((char *));
-extern void il_set_error_flag		 PROTO ((int));
-extern void il_history			 PROTO ((int));
+extern void il_kill_to_end_of_line PROTO (());
+extern void il_just_one_space PROTO (());
+extern void il_delete_horizontal_space PROTO (());
+extern void il_downcase_word PROTO (());
+extern void il_upcase_word PROTO (());
+extern void il_capitalize_word PROTO (());
+extern void il_set_static_text PROTO ((char *));
+extern void il_insert_text PROTO ((char *));
+extern void il_update_point PROTO (());
+extern void il_update PROTO (());
+extern void il_full_update PROTO (());
+extern int il_get_contents PROTO ((char **));
+extern void il_message PROTO ((char *));
+extern void il_set_error_flag PROTO ((int));
+extern void il_history PROTO ((int));
 
 
 #endif  /* _GIT_INPUTLINE_H */
