@@ -1,7 +1,6 @@
-/* misc.c -- miscelaneous functions used in git/gitps/gitview. I'll add here
-   anything that won't fit anywhere else. */
+/* misc.c -- Miscelaneous functions used in git/gitps/gitview.  */
 
-/* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1993, 1994, 1995, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,6 +34,11 @@
 #include <signal.h>
 
 #include <sys/types.h>
+
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
 #include <pwd.h>
 
 #ifdef HAVE_UNISTD_H
@@ -74,6 +78,20 @@ char *month_name[] =
 /* The pointer to the head of the file type attributes list.  */
 file_type_info_t *fti_head = NULL;
 
+extern void fatal PROTO ((char *));
+
+
+void
+display_exit_message(signame)
+    char *signame;
+{
+    struct tm *time = get_local_time();
+
+    fprintf(stderr, "%s %d %2d:%02d:%02d %s[%d]: exiting on %s signal\n",
+	    month_name[time->tm_mon], time->tm_mday, time->tm_hour,
+	    time->tm_min, time->tm_sec, program, (int)pid, signame);
+}
+
 
 RETSIGTYPE
 fatal_signal(signum)
@@ -101,7 +119,6 @@ fatal_signal(signum)
 
 	    display_exit_message("SEGV");
 	    goto ask_report;
-	    break;
 
 	default:
 
@@ -109,7 +126,7 @@ fatal_signal(signum)
 		    "%s: got a stupid signal (%d). Unless it was a joke ...\n",
 		    program, signum);
 	  ask_report:
-	    fprintf(stderr, "%s: please report to tudor@chang.pub.ro\n",
+	    fprintf(stderr, "%s: please report to tudor@cs.unh.edu\n",
 		    program);
 	    break;
     }
@@ -118,24 +135,13 @@ fatal_signal(signum)
 }
 
 
-void
-display_exit_message(signame)
-    char *signame;
-{
-    struct tm *time = get_local_time();
-
-    fprintf(stderr, "%s %d %2d:%02d:%02d %s[%d]: exiting on %s signal\n",
-	    month_name[time->tm_mon], time->tm_mday, time->tm_hour,
-	    time->tm_min, time->tm_sec, program, (int)pid, signame);
-}
-
-
 void
 configuration_fatal_error(configfile)
     char *configfile;
 {
     fprintf(stderr, "%s: installation problem: \n", program);
-    fprintf(stderr, "%s: cannot find configuration file '%s'.\n\n", program, configfile);
+    fprintf(stderr, "%s: cannot find configuration file '%s'.\n\n",
+	    program, configfile);
 }
 
 
@@ -143,57 +149,59 @@ void
 configuration_warning(configfile)
     char *configfile;
 {
-    fprintf(stderr, "\n%s: cannot open configuration file '%s'.\n",
+    fprintf(stderr,
+	    "\n%s: Cannot open configuration file '%s'.\n",
 	    program, configfile);
-    fprintf(stderr, "%s: See the manual page for details.\n", program);
     fprintf(stderr,
-	   "%s: If the TERM environment variable is, let's say, vt102, your\n",
-	    program);
-    fprintf(stderr, "%s: configuration file name should be '.gitrc.vt102'.\n",
+	    "%s: See the info documentation for details.\n",
 	    program);
     fprintf(stderr,
-	   "%s: You can copy a configuration file in your home directory to\n",
+	   "%s: If the TERM environment variable is, say, vt102, your\n",
+	    program);
+    fprintf(stderr,
+	    "%s: configuration file name is '.gitrc.vt102'.\n",
+	    program);
+    fprintf(stderr,
+	   "%s: You can copy a configuration file in your home directory\n",
+	    program);
+    fprintf(stderr,
+	    "%s: and modify it in order to overwrite the default one.\n",
+	    program);
+    fprintf(stderr,
+	    "%s: Try modifying '.gitrc.generic'...\n\n",
 	    program);
-    fprintf(stderr, "%s: overwrite the default one. \n", program);
-    fprintf(stderr, "%s: Try modifying '.gitrc.generic'...\n\n", program);
 }
 
 
 void
 common_configuration_init()
 {
-    int config_ok;
-
+    /* Load the global .gitrc.common file.  */
     char *configfile = xmalloc(strlen(term) + 1 + strlen(CONFIGFILE_PREFIX) +
-			       strlen(tty_type) + 1);
+			       sizeof("common") + 1);
     strcpy(configfile, term);
     strcat(configfile, CONFIGFILE_PREFIX);
     strcat(configfile, "common");
 
-    config_ok = configuration_init(configfile);
-
-    if (!config_ok)
+    if (configuration_init(configfile) == 0)
     {
+	/* Give up if global .gitrc.common is not found.  */
 	configuration_fatal_error(configfile);
 	exit(1);
     }
 }
 
 
-void
+int
 specific_configuration_init()
 {
-    int config_ok;
-
     char *configfile = xmalloc(strlen(home) + 1 + strlen(CONFIGFILE_PREFIX) +
 			       strlen(tty_type) + 1);
     strcpy(configfile, home);
     strcat(configfile, CONFIGFILE_PREFIX);
     strcat(configfile, tty_type);
 
-    config_ok = configuration_init(configfile);
-
-    if (!config_ok)
+    if (configuration_init(configfile) == 0)
     {
 	xfree(configfile);
 	configfile = xmalloc(strlen(term) + 1 + strlen(CONFIGFILE_PREFIX) +
@@ -202,30 +210,30 @@ specific_configuration_init()
 	strcat(configfile, CONFIGFILE_PREFIX);
 	strcat(configfile, tty_type);
 
-	config_ok = configuration_init(configfile);
-
-	if (!config_ok)
+	if (configuration_init(configfile) == 0)
 	{
 	    configuration_warning(configfile);
 
 	    xfree(configfile);
 	    configfile = xmalloc(strlen(term) + 1 + strlen(CONFIGFILE_PREFIX) +
-				 10 + 1);
+				 sizeof("generic") + 1);
 	    strcpy(configfile, term);
 	    strcat(configfile, CONFIGFILE_PREFIX);
 	    strcat(configfile, "generic");
 
-	    config_ok = configuration_init(configfile);
-
-	    if (!config_ok)
+	    if (configuration_init(configfile) == 0)
 	    {
 		configuration_fatal_error(configfile);
 		exit(1);
 	    }
+
+	    return 0;
 	}
     }
 
     xfree(configfile);
+
+    return 1;
 }
 
 
@@ -293,8 +301,12 @@ get_flag_var(var_name, default_value)
 
     if (data)
     {
-	if (strcmp(data, "ON")  == 0) return 1;
-	if (strcmp(data, "OFF") == 0) return 0;
+	if (strcmp(data, "ON")  == 0)
+	    return 1;
+
+	if (strcmp(data, "OFF") == 0)
+	    return 0;
+
 	fprintf(stderr, "%s: invalid %s (%s).\n", program, var_name, data);
 	return default_value;
     }
@@ -344,35 +356,60 @@ get_colorset_var(charset, colorset_name, fields_no)
 }
 
 
+/*
+ * Minimize the path, that is, convert paths like /etc/./././///./ into
+ * their cleaner form: /etc.  Stolen from the Thix kernel.  Enhanced to
+ * minimize paths like /usr/local/bin/../lib as well.
+ */
+
 char *
-clear_path(path)
+minimize_path(path)
     char *path;
 {
     char *cpath = path;
     char *opath = path;
 
-    if (*opath == 0)
-        return path;
-
     if (*opath == '/')
 	*cpath++ = *opath++;
+    else
+	fatal("relative path encounter");
 
     while (*opath)
     {
 	while (*opath == '/' ||
-	       (*opath == '.' && (*(opath + 1) == '/' || *(opath + 1) == 0)))
+	       (*opath == '.' &&
+		(*(opath + 1) == '/' || *(opath + 1) == '\0')))
 	    opath++;
 
+	if (*opath == '.' && *(opath + 1) == '.' &&
+	    (*(opath + 2) == '/' || *(opath + 2) == '\0'))
+	{
+	    /* This is something like /usr/local/bin/.. and we are
+	       going to remove the trailing .. along with the
+	       directory that no longer meakes sense: bin.  */
+
+	    /* Check for /..  and do nothing if this is the case.  */
+	    if (cpath - 1 != path)
+	    {
+		for (cpath -= 2; *cpath != '/'; cpath--);
+		cpath++;
+	    }
+
+	    opath += 2;
+	    continue;
+	}
+
 	while (*opath && *opath != '/')
 	    *cpath++ = *opath++;
 
 	if (*opath)
 	    *cpath++ = '/';
     }
-    if (*(cpath - 1) == '/' && cpath - path > 1)
-        cpath--;
 
-    *cpath = 0;
+    if (*(cpath - 1) == '/' && cpath - path > 1)
+	cpath--;
+
+    *cpath = '\0';
     return path;
 }
 
@@ -418,9 +455,9 @@ truncate_long_name(name, dest, len)
     char *name, *dest;
     size_t len;
 {
-    size_t name_len;
+    size_t name_len = strlen(name);
 
-    if ((name_len = strlen(name)) > len)
+    if (name_len > len)
     {
 	dest[0] = dest[1] = dest[2] = '.';
 	memcpy(dest + 3, name + name_len - len + 3, len - 3);
@@ -436,21 +473,21 @@ truncate_string(path, temppath, len)
     char *temppath;
     size_t len;
 {
-    truncate_long_name(path, temppath, len);
-    temppath[min(len, strlen(path))] = 0;
+    truncate_long_name(path, temppath, len - 1);
+    temppath[min(len - 1, strlen(path))] = '\0';
     return temppath;
 }
 
 
 int
-get_file_length(handle)
-    int handle;
+get_file_length(fd)
+    int fd;
 {
-    int tmp, length;
+    int current, length;
 
-    tmp    = lseek(handle, 0, SEEK_CUR);
-    length = lseek(handle, 0, SEEK_END);
-    lseek(handle, tmp, SEEK_SET);
+    current = lseek(fd, 0, SEEK_CUR);
+    length  = lseek(fd, 0, SEEK_END);
+    lseek(fd, current, SEEK_SET);
     return length;
 }
 
@@ -466,19 +503,112 @@ get_local_time()
 }
 
 
-char *
-xbasename(name)
-    char *name;
+#ifndef HAVE_PUTENV
+#ifndef HAVE_SETENV
+#ifdef NeXT
+/*
+ * m68k-next-nextstep (NeXT - Next Step 3.2) seems to lack putenv().
+ * I have included /NextDeveloper/Source/GNU/debug/gdb/gdb/putenv.c
+ * here.  Paul Nevai <nevai@ops.mps.ohio-state.edu> used it to make
+ * git compile on the NeXT.
+ */
+
+/*
+ * Path: hoptoad!pacbell!ames!ll-xn!mit-eddie!uw-beaver!ssc-vax!uvicctr!tholm
+ * From: tholm@uvicctr.UUCP (Terrence W. Holm)
+ * Newsgroups: comp.os.minix
+ * Subject: putenv(3)
+ * Message-ID: <395@uvicctr.UUCP>
+ * Date: 5 May 88 06:40:52 GMT
+ * Organization: University of Victoria, Victoria B.C. Canada
+ *
+ * EFTH Minix report #2  - May 1988 -  putenv(3)
+ *
+ * This is an implementation of putenv(3) that we
+ * wrote for Minix. Please consider this a public
+ * domain program.
+ *
+ */
+
+extern char **environ;
+
+/****************************************************************/
+/*								*/
+/*      int							*/
+/*	putenv( entry )						*/
+/*								*/
+/*		The "entry" should follow the form 		*/
+/*		"NAME=VALUE". This routine will search the 	*/
+/*		user environment for "NAME" and replace its 	*/
+/*		value with "VALUE".				*/
+/*								*/
+/*		Note that "entry" is not copied, it is used 	*/
+/*		as the environment entry. This means that it 	*/
+/*		must not be unallocated or otherwise modifed 	*/
+/*		by the caller, unless it is replaced by a 	*/
+/*		subsequent putenv().				*/
+/*								*/
+/*		If the name is not found in the environment, 	*/
+/*		then a new vector of pointers is allocated, 	*/
+/*		"entry" is put at the end and the global 	*/
+/*		variable "environ" is updated.			*/
+/*								*/
+/*		This function normally returns NULL, but -1	*/
+/*		is returned if it can not allocate enough 	*/
+/*		space using malloc(3), or "entry" does not	*/
+/*		contain a '='.					*/
+/*								*/
+/****************************************************************/
+
+
+int
+putenv(entry)
+    char *entry;
 {
-    char *base;
-    size_t len = strlen(name);
+    unsigned length;
+    unsigned size;
+    char     *temp;
+    char     **p;
+    char     **new_environ;
 
-    if (name[len - 1] == '/')
-	name[len - 1] = '\0';
+    /* Find the length of the "NAME=".  */
+    temp = strchr(entry,'=');
 
-    base = strrchr(name, '/');
-    return base ? base + 1 : name;
+    if (temp == 0)
+	return -1;
+
+    length = (unsigned) (temp - entry + 1);
+
+    /* Scan through the environment looking for "NAME="  */
+    for (p = environ; *p != 0 ; p++)
+	if (strncmp(entry, *p, length) == 0)
+	{
+	    *p = entry;
+	    return 0;
+	}
+
+
+    /*  The name was not found, build a bigger environment  */
+    size = p - environ;
+    new_environ = (char **)malloc((size + 2) * sizeof(char *));
+
+    if (new_environ == (char **)NULL)
+	return -1;
+
+    memcpy((char *)new_environ, (char *)environ, size * sizeof(char *));
+
+    new_environ[size]   = entry;
+    new_environ[size+1] = NULL;
+
+    environ = new_environ;
+
+    return 0;
 }
+#else
+/* I guess we are simply out of luck.  #error ?  */
+#endif /* NeXT */
+#endif /* HAVE_SETENV */
+#endif /* HAVE_PUTENV */
 
 
 /*
@@ -530,35 +660,34 @@ void
 get_file_type_info()
 {
     char *contents[3];
-    unsigned char regexp[80];
+    char pattern[80];
     int brightness, foreground, background;
     file_type_info_t *previous = NULL, *fti, *fti_head1 = NULL;
 
 
     for (;;)
     {
-        configuration_getvarinfo((char *)regexp, contents, 3, NO_SEEK);
+	configuration_getvarinfo(pattern, contents, 3, NO_SEEK);
 
-        if (*regexp == 0)
-            break;
+	if (*pattern == '\0')
+	    break;
 
-        if (contents[0])
-            foreground = tty_get_color_index(contents[0]);
-        else
+	if (contents[0])
+	    foreground = tty_get_color_index(contents[0]);
+	else
 	    foreground = -1;
 
-        if (contents[1])
-            background = tty_get_color_index(contents[1]);
-        else
+	if (contents[1])
+	    background = tty_get_color_index(contents[1]);
+	else
 	    background = -1;
 
-        if (contents[2])
-            brightness = tty_get_color_index(contents[2]);
-        else
+	if (contents[2])
+	    brightness = tty_get_color_index(contents[2]);
+	else
 	    brightness = -1;
 
-	/* Insert the just obtained file type entry into the list.  */
-
+	/* Insert the file type entry just obtained into the list.  */
 	fti = (file_type_info_t *)xmalloc(sizeof(file_type_info_t));
 
 	if (fti_head1 == NULL)
@@ -566,13 +695,13 @@ get_file_type_info()
 	else
 	    previous->next = fti;
 
-	fti->regexp     = xstrdup(regexp);
+	fti->pattern    = xstrdup(pattern);
 	fti->foreground = foreground;
 	fti->background = background;
 	fti->brightness = brightness;
 	fti->next       = NULL;
 
-        previous = fti;
+	previous = fti;
     }
 
     /* Fixed by Marian Ciobanu <ciobi@liis.sfos.ro>, October 24, 1995.  */
@@ -581,7 +710,7 @@ get_file_type_info()
 	{
 	    previous->next = fti_head;
 	    fti_head = fti_head1;
-        }
+	}
 	else
 	    fti_head = fti_head1;
 }
