@@ -1,4 +1,4 @@
-/* signal.c -- a file which should cantain some *REAL* signals management. */
+/* signal.c -- A file which should cantain some *REAL* signals management.  */
 
 /* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
 
@@ -106,9 +106,22 @@ signals(status)
     int status;
 {
     signals_status = status;
-    signal(SIGTERM, status == ON ? fatal_signal : SIG_IGN);
-    signal(SIGQUIT, status == ON ? fatal_signal : SIG_IGN);
-    signal(SIGINT , status == ON ? user_panic   : SIG_IGN);
+
+
+    /* Don't use conditional expressions here, the Alpha OSF 4.0 C
+       compiler will complain.  */
+    if (status == ON)
+    {
+	signal(SIGTERM, fatal_signal);
+	signal(SIGQUIT, fatal_signal);
+	signal(SIGINT , user_panic);
+    }
+    else
+    {
+	signal(SIGTERM, SIG_IGN);
+	signal(SIGQUIT, SIG_IGN);
+	signal(SIGINT , SIG_IGN);
+    }
 }
 
 
