@@ -1,4 +1,4 @@
-/* stat.h -- safely defined stat constants.  */
+/* stat.h -- Safely defined stat constants.  */
 
 /* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
 
@@ -132,6 +132,10 @@
 #define S_IRWXG         0000070
 #endif
 
+#ifndef S_IRWXO
+#define S_IRWXO         0000007
+#endif
+
 /* Finally ... :-( */
 
 
