@@ -1,4 +1,4 @@
-/* tilde.c -- Tilde expansion code (~/foo := $HOME/foo). */
+/* tilde.c -- Tilde expansion code (~/foo := $HOME/foo).  */
 
 /* Copyright (C) 1988,1989 Free Software Foundation, Inc.
 
@@ -31,6 +31,7 @@
 #endif /* HAVE_STDLIB_H */
 
 #include <sys/types.h>
+#include <pwd.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -40,7 +41,6 @@
 #include "xstring.h"
 #include "tilde.h"
 
-#include <pwd.h>
 
 #if defined (USG) && !defined (HAVE_GETPW_DECLS)
 extern struct passwd *getpwuid (), *getpwnam ();
@@ -55,11 +55,11 @@ extern char *strcpy ();
 #endif /* !savestring */
 
 #if !defined (NULL)
-#  if defined (STDC_HEADERS)
+#  if defined (__STDC__)
 #    define NULL ((void *) 0)
 #  else
 #    define NULL 0x0
-#  endif /* !STDC_HEADERS */
+#  endif /* !__STDC__ */
 #endif /* !NULL */
 
 
@@ -173,7 +173,7 @@ tilde_expand (string)
 
       /* Copy the skipped text into the result. */
       if ((result_index + start + 1) > result_size)
-	result = (char *)xrealloc (result, 1 + (result_size += (start + 20)));
+	result = xrealloc (result, 1 + (result_size += (start + 20)));
 
       strncpy (result + result_index, string, start);
       result_index += start;
@@ -190,21 +190,21 @@ tilde_expand (string)
 	break;
 
       /* Expand the entire tilde word, and copy it into RESULT. */
-      tilde_word = (char *)xmalloc (1 + end);
+      tilde_word = xmalloc (1 + end);
       strncpy (tilde_word, string, end);
       tilde_word[end] = '\0';
       string += end;
 
       expansion = tilde_expand_word (tilde_word);
-      free (tilde_word);
+      xfree (tilde_word);
 
       len = strlen (expansion);
       if ((result_index + len + 1) > result_size)
-	result = (char *)xrealloc (result, 1 + (result_size += (len + 20)));
+	result = xrealloc (result, 1 + (result_size += (len + 20)));
 
       strcpy (result + result_index, expansion);
       result_index += len;
-      free (expansion);
+      xfree (expansion);
     }
 
   result[result_index] = '\0';
@@ -219,89 +219,89 @@ tilde_expand_word (filename)
      char *filename;
 {
   char *dirname;
+  char *temp_name;
 
-  dirname = filename ? savestring (filename) : (char *)NULL;
+  if (filename == (char *)0)
+    return ((char *)NULL);
 
-  if (dirname && *dirname == '~')
+  dirname = savestring (filename);
+
+  if (*dirname != '~')
+    return (dirname);
+
+  if (!dirname[1] || dirname[1] == '/')
     {
-      char *temp_name;
-      if (!dirname[1] || dirname[1] == '/')
+      /* Prepend $HOME to the rest of the string. */
+      char *temp_home = (char *)getenv ("HOME");
+      int home_len;
+
+      /* If there is no HOME variable, look up the directory in
+	 the password database. */
+      if (!temp_home)
 	{
-	  /* Prepend $HOME to the rest of the string. */
-	  char *temp_home = (char *)getenv ("HOME");
+	  struct passwd *entry;
 
-	  /* If there is no HOME variable, look up the directory in
-	     the password database. */
-	  if (!temp_home)
+	  entry = getpwuid (getuid ());
+	  if (entry)
+	    temp_home = entry->pw_dir;
+	}
+
+      home_len = temp_home ? strlen (temp_home) : 0;
+      temp_name = xmalloc (1 + strlen (dirname + 1) + home_len);
+			     
+      if (temp_home)
+	strcpy (temp_name, temp_home);
+      strcpy (temp_name + home_len, dirname + 1);
+      xfree (dirname);
+      dirname = temp_name;
+    }
+  else
+    {
+      char *username;
+      struct passwd *user_entry;
+      int i, len;
+
+      username = xmalloc (strlen (dirname));
+      for (i = 1; dirname[i] && dirname[i] != '/'; i++)
+	username[i - 1] = dirname[i];
+      username[i - 1] = '\0';
+
+      if ((user_entry = getpwnam (username)) == (struct passwd *)0)
+	{
+	  /* If the calling program has a special syntax for
+	     expanding tildes, and we couldn't find a standard
+	     expansion, then let them try. */
+	  if (tilde_expansion_failure_hook)
 	    {
-	      struct passwd *entry;
+	      char *expansion;
 
-	      entry = getpwuid (getuid ());
-	      if (entry)
-		temp_home = entry->pw_dir;
+	      expansion = (*tilde_expansion_failure_hook) (username);
+
+	      if (expansion)
+		{
+		  len = strlen (expansion);
+		  temp_name = xmalloc (1 + len + strlen (dirname + i));
+		  strcpy (temp_name, expansion);
+		  strcpy (temp_name + len, dirname + i);
+		  xfree (expansion);
+		  xfree (dirname);
+		  dirname = temp_name;
+		}
 	    }
-
-	  temp_name = xmalloc (1 + strlen (&dirname[1])
-				 + (temp_home ? strlen (temp_home) : 0));
-	  temp_name[0] = '\0';
-	  if (temp_home)
-	    strcpy (temp_name, temp_home);
-	  strcat (temp_name, dirname + 1);
-	  free (dirname);
-	  dirname = temp_name;
+	  /* We shouldn't report errors. */
 	}
       else
 	{
-	  char u_name[257];
-	  struct passwd *user_entry;
-	  char *username;
-	  int i, c;
-
-	  username = u_name;
-	  for (i = 1; (c = dirname[i]); i++)
-	    {
-	      if (c == '/')
-		break;
-	      else
-		username[i - 1] = c;
-	    }
-	  username[i - 1] = '\0';
-
-	  if (!(user_entry = getpwnam (username)))
-	    {
-	      /* If the calling program has a special syntax for
-		 expanding tildes, and we couldn't find a standard
-		 expansion, then let them try. */
-	      if (tilde_expansion_failure_hook)
-		{
-		  char *expansion;
-
-		  expansion = (*tilde_expansion_failure_hook) (username);
-
-		  if (expansion)
-		    {
-		      temp_name = xmalloc (1 + strlen (expansion)
-						  + strlen (&dirname[i]));
-		      strcpy (temp_name, expansion);
-		      strcat (temp_name, &dirname[i]);
-		      free (expansion);
-		      free (dirname);
-		      dirname = temp_name;
-		    }
-		}
-	      /* We shouldn't report errors. */
-	    }
-	  else
-	    {
-	      temp_name = xmalloc (1 + strlen (user_entry->pw_dir)
-				     + strlen (&dirname[i]));
-	      strcpy (temp_name, user_entry->pw_dir);
-	      strcat (temp_name, &dirname[i]);
-	      free (dirname);
-	      dirname = temp_name;
-	    }
-	    endpwent ();
+	  len = strlen (user_entry->pw_dir);
+	  temp_name = xmalloc (1 + len + strlen (dirname + i));
+	  strcpy (temp_name, user_entry->pw_dir);
+	  strcpy (temp_name + len, dirname + i);
+	  xfree (dirname);
+	  dirname = temp_name;
 	}
+      endpwent ();
+      xfree (username);
     }
+
   return (dirname);
 }
