@@ -1,8 +1,6 @@
-/* tty.c -- the tty management file. This includes termcap / terminfo
-   initialization, colors, keys and functions to switch terminals
-   between canonical and noncanonical modes.  */
+/* tty.c -- The tty management file.  */
 
-/* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1993-1997 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,6 +32,11 @@
 #endif /* !HAVE_STDLIB_H */
 
 #include <sys/types.h>
+
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
 #include <ctype.h>
 #include "file.h"
 #include <sys/ioctl.h>
@@ -72,12 +75,12 @@
    defines these.  I think it's safer because I couldn't find 2
    similar curses.h files in the entire world...  */
 
-int tputs     PROTO ((const char *__string, int __nlines, int (*outfun)()));
-int tgetent   PROTO ((void *__buffer, const char *__termtype));
-char *tgetstr PROTO ((const char *__name, char **__area));
-int tgetnum   PROTO ((const char *__name));
-int tgetflag  PROTO ((const char *__name));
-char *tgoto   PROTO ((const char *__cstring, int __hpos, int __vpos));
+extern int tputs PROTO ((const char *__string, int __nlines, int (*outfun)()));
+extern int tgetent PROTO ((void *__buffer, const char *__termtype));
+extern char *tgetstr PROTO ((const char *__name, char **__area));
+extern int tgetnum PROTO ((const char *__name));
+extern int tgetflag PROTO ((const char *__name));
+extern char *tgoto PROTO ((const char *__cstring, int __hpos, int __vpos));
 
 
 #define TTY_INPUT       0
@@ -93,8 +96,9 @@ static int vcs_read_ok;
 static int vcs_is_bw;
 #endif
 
-/* If tty_kbdmode == 1, single characters are inserted in the linked list.
-   This feature is used by gitps (it has no command line).  */
+/* If tty_kbdmode == 1, single character key sequences are inserted
+   into the linked list.  This feature is used by gitps which has no
+   command line.  */
 static int tty_kbdmode;
 
 #ifdef HAVE_POSIX_TTY
@@ -120,10 +124,8 @@ static struct ltchars new_ltarg;
 #endif /* HAVE_SYSTEMV_TTY */
 #endif /* HAVE_POSIX_TTY */
 
-static int lines, columns;
-
-static tty_last_char_flag;
-
+static int tty_columns, tty_lines;
+static int tty_last_char_flag;
 static int tty_cursor_x;
 static int tty_cursor_y;
 
@@ -230,7 +232,9 @@ tty_key_t default_key;
 static char tty_cache[TTY_CACHE_SIZE];
 static unsigned int tty_index;
 
+#ifndef HAVE_LINUX
 static char term_buf[2048];
+#endif
 static char vt100[] = "vt100";
 
 /* The terminal mode. TTY_CANONIC at the begining.  */
@@ -244,9 +248,9 @@ char *UP;
 
 #ifdef HAVE_LINUX
 speed_t ospeed;
-#else   /* !HAVE_LINUX */
+#else /* !HAVE_LINUX */
 short ospeed;
-#endif  /* !HAVE_LINUX */
+#endif /* !HAVE_LINUX */
 
 
 /* A structure describing some attributes we need to know about each
@@ -263,27 +267,29 @@ typedef struct
 
     int required;	/* This capability is required.  */
     char *symbol;	/* The human readable form of the key name.  */
-} TTY_CAPABILITY;
+} tty_capability_t;
 
 
-#define TTY_USED_CAPABILITIES   34
-#define TTY_FIRST_SYMBOL_KEY    13
+#define TTY_CAPABILITIES_USED   36
+#define TTY_FIRST_SYMBOL_KEY    15
 
-static TTY_CAPABILITY tty_capability[TTY_USED_CAPABILITIES] =
+static tty_capability_t tty_capability[TTY_CAPABILITIES_USED] =
 {
-    { "me", NULL, 0, 0, NULL },		/* turn off all attributes */
-    { "mr", NULL, 0, 0, NULL },		/* turn on reverse video mode */
-    { "md", NULL, 0, 0, NULL },		/* turn on bold */
-    { "vi", NULL, 0, 0, NULL },		/* make the cursor invisible */
-    { "ve", NULL, 0, 0, NULL },		/* make the cursor appear normal */
-    { "cl", NULL, 0, 1, NULL },		/* clear screen & home the cursor */
-    { "cm", NULL, 0, 1, NULL },		/* move the cursor */
-    { "pc", NULL, 0, 0, NULL },		/* padding character */
-    { "up", NULL, 0, 0, NULL },		/* up one line */
-    { "le", NULL, 0, 0, NULL },		/* move left one space */
-    { "so", NULL, 0, 0, NULL },		/* enter standout mode.  */
-    { "sg", NULL, 0, 0, NULL },		/* this is a magic-cookie terminal.  */
-    { "ms", NULL, 0, 0, NULL },		/* safe to move in standout mode.  */
+    { "me", NULL, 0, 0, NULL },		/* Turn off all attributes.  */
+    { "mr", NULL, 0, 0, NULL },		/* Turn on reverse video mode.  */
+    { "md", NULL, 0, 0, NULL },		/* Turn on bold.  */
+    { "vi", NULL, 0, 0, NULL },		/* Make the cursor invisible.  */
+    { "ve", NULL, 0, 0, NULL },		/* Make the cursor appear normal.  */
+    { "cl", NULL, 0, 1, NULL },		/* Clear screen & home the cursor.  */
+    { "cm", NULL, 0, 1, NULL },		/* Move the cursor.  */
+    { "pc", NULL, 0, 0, NULL },		/* Padding character.  */
+    { "up", NULL, 0, 0, NULL },		/* Up one line.  */
+    { "le", NULL, 0, 0, NULL },		/* Move left one space.  */
+    { "so", NULL, 0, 0, NULL },		/* Enter standout mode.  */
+    { "sg", NULL, 0, 0, NULL },		/* This is a magic-cookie terminal.  */
+    { "ms", NULL, 0, 0, NULL },		/* Safe to move in standout mode.  */
+    { "co", NULL, 0, 0, NULL },		/* The number of columns.  */
+    { "li", NULL, 0, 0, NULL },		/* The number of lines.  */
     { "ku", NULL, 0, 0, "UP" },		/* (UP) */
     { "kd", NULL, 0, 0, "DOWN" },	/* (DOWN) */
     { "kr", NULL, 0, 0, "RIGHT" },	/* (RIGHT) */
@@ -291,7 +297,7 @@ static TTY_CAPABILITY tty_capability[TTY_USED_CAPABILITIES] =
     { "kI", NULL, 0, 0, "INS" },	/* (INS) */
     { "kD", NULL, 0, 0, "DEL" },	/* (DEL) */
     { "kh", NULL, 0, 0, "HOME" },	/* (HOME) */
-    { "kH", NULL, 0, 0, "END" },	/* (END) */
+    { "@7", NULL, 0, 0, "END" },	/* (END) */
     { "kP", NULL, 0, 0, "PGUP" },	/* (PGUP) */
     { "kN", NULL, 0, 0, "PGDOWN" },	/* (PGDOWN) */
     { "k0", NULL, 0, 0, "F0" },		/* (F0) */
@@ -322,9 +328,11 @@ static TTY_CAPABILITY tty_capability[TTY_USED_CAPABILITIES] =
 #define TTY_STANDOUT_ON		(tty_capability[10].string)
 #define TTY_MAGIC_COOKIE	(tty_capability[11].integer)
 #define TTY_MS_FLAG		(tty_capability[12].integer)
+#define TTY_COLUMNS		(tty_capability[13].integer)
+#define TTY_LINES		(tty_capability[14].integer)
 
 
-/* Some more nice aliases.  */
+/* Some more nice aliases...  */
 #define TTY_ATTRIBUTES_OFF_NAME (tty_capability[0].name)
 #define TTY_REVERSE_ON_NAME     (tty_capability[1].name)
 #define TTY_BRIGHT_ON_NAME      (tty_capability[2].name)
@@ -338,6 +346,8 @@ static TTY_CAPABILITY tty_capability[TTY_USED_CAPABILITIES] =
 #define TTY_STANDOUT_ON_NAME	(tty_capability[10].name)
 #define TTY_MAGIC_COOKIE_NAME	(tty_capability[11].name)
 #define TTY_MS_FLAG_NAME	(tty_capability[12].name)
+#define TTY_COLUMNS_NAME	(tty_capability[13].name)
+#define TTY_LINES_NAME		(tty_capability[14].name)
 
 
 #ifdef HAVE_LIBTERMCAP
@@ -353,15 +363,15 @@ static char term_env[]      = "TERMINFO";
 #endif  /* !HAVE_LIBTERMCAP */
 
 
-static void tty_io_goto		PROTO ((int, int));
-static void tty_io_foreground 	PROTO ((int));
-static void tty_io_background 	PROTO ((int));
-static void tty_io_brightness 	PROTO ((int));
-static void tty_io_reversevid 	PROTO ((int));
-static void tty_io_colors	PROTO ((unsigned char));
+static void tty_io_goto PROTO ((int, int));
+static void tty_io_foreground PROTO ((int));
+static void tty_io_background PROTO ((int));
+static void tty_io_brightness PROTO ((int));
+static void tty_io_reversevid PROTO ((int));
+static void tty_io_colors PROTO ((unsigned char));
 
 
-void fatal PROTO ((char *));
+extern void fatal PROTO ((char *));
 
 
 /* ANSI colors. OFF & ON are here because we need them when we read the
@@ -508,18 +518,18 @@ tty_startup(last_char)
 {
     /* Fail if stdin or stdout are not real terminals.  */
     if (!isatty(TTY_INPUT) || !isatty(TTY_OUTPUT))
-	fatal("only standard error can be redirected");
+	fatal("only stderr can be redirected");
 
     /* Get calloc-ed memory for tty_scr & tty_atr.  */
-    tty_scr = (unsigned char *)xcalloc(columns * lines,
+    tty_scr = (unsigned char *)xcalloc(tty_columns * tty_lines,
 				       sizeof(unsigned char));
-    tty_atr = (unsigned char *)xcalloc(columns * lines,
+    tty_atr = (unsigned char *)xcalloc(tty_columns * tty_lines,
 				       sizeof(unsigned char));
 
     /* Get calloc-ed memory for tty_prev_scr & tty_prev_atr.  */
-    tty_prev_scr = (unsigned char *)xcalloc(columns * lines,
+    tty_prev_scr = (unsigned char *)xcalloc(tty_columns * tty_lines,
 					    sizeof(unsigned char));
-    tty_prev_atr = (unsigned char *)xcalloc(columns * lines,
+    tty_prev_atr = (unsigned char *)xcalloc(tty_columns * tty_lines,
 					    sizeof(unsigned char));
 
     /* Store the terminal settings in old_term. it will be used to restore
@@ -718,8 +728,8 @@ tty_set_mode(mode)
 			 ~(ECHO | CRMOD | XTABS | ALLDELAY | TILDE)) | CBREAK);
 	new_targ.t_intrc   = key_INTERRUPT;     /* Ctrl-G */
 	new_targ.t_quitc   = key_INTERRUPT;     /* Ctrl-G */
-	new_targ.t_stopc   = CDISABLE;		/* Ctrl-G */
-	new_targ.t_startc  = CDISABLE;		/* Ctrl-G */
+	new_targ.t_stopc   = CDISABLE;
+	new_targ.t_startc  = CDISABLE;
 	new_targ.t_eofc    = CDISABLE;
 	new_targ.t_brkc    = CDISABLE;
 	new_ltarg.t_lnextc = CDISABLE;
@@ -738,12 +748,19 @@ tty_set_mode(mode)
 
 /* Try to make sure the terminal is not locked.  */
 #ifdef TCXONC
-      ioctl (TTY_OUTPUT, TCXONC, 1);
+#ifdef __QNX__
+	{
+	    int value = 1;
+	    ioctl (TTY_OUTPUT, TCXONC, &value);
+	}
+#else
+	ioctl(TTY_OUTPUT, TCXONC, 1);
+#endif
 #endif
 
 #ifndef APOLLO
 #ifdef TIOCSTART
-      ioctl (TTY_OUTPUT, TIOCSTART, 0);
+	ioctl(TTY_OUTPUT, TIOCSTART, 0);
 #endif
 #endif
 
@@ -829,7 +846,7 @@ tty_writec(c)
 void
 tty_io_clear()
 {
-    tputs(TTY_CLEAR_SCREEN, lines, tty_writec);
+    tputs(TTY_CLEAR_SCREEN, tty_lines, tty_writec);
     tty_flush();
 }
 
@@ -866,7 +883,7 @@ tty_exit(screen)
 
 /*
  * Converts a key sequence from the human readable, '^' based form into a
- * computer usable form.
+ * machine usable form.
  */
 
 char *
@@ -887,7 +904,14 @@ tty_key_convert(key_seq)
 	if (*second == '^')
 	{
 	    if (*++second)
+	    {
+		/* ^G is the interrupt character and ^Z the suspend one.
+		   They are not allowed in key sequences.  */
+		if (toupper(*second) == 'G' || toupper(*second) == 'Z')
+		    return NULL;
+
 		*first++ = key_ctrl_table[(*second++ & 0x7F) - ' '];
+	    }
 	    else
 		return NULL;
 	}
@@ -969,8 +993,8 @@ tty_update()
 	tty_flush();
 
     /* Synchronize the screen copies.  */
-    memcpy(tty_prev_scr, tty_scr, columns * lines * sizeof(char));
-    memcpy(tty_prev_atr, tty_atr, columns * lines * sizeof(char));
+    memcpy(tty_prev_scr, tty_scr, tty_columns * tty_lines * sizeof(char));
+    memcpy(tty_prev_atr, tty_atr, tty_columns * tty_lines * sizeof(char));
 }
 
 
@@ -1000,7 +1024,7 @@ tty_puts(buf, length)
     char *buf;
     size_t length;
 {
-    int tty_offset = (tty_cursor_y * columns) + tty_cursor_x;
+    int tty_offset = (tty_cursor_y * tty_columns) + tty_cursor_x;
 
     memcpy(tty_scr + tty_offset, buf, length);
     memset(tty_atr + tty_offset, tty_current_attribute, length);
@@ -1016,9 +1040,9 @@ tty_puts(buf, length)
 
 int
 tty_putc(c)
-    unsigned char c;
+    char c;
 {
-    return tty_puts(&c, 1);
+    return tty_puts(&c, (size_t)1);
 }
 
 
@@ -1061,10 +1085,10 @@ tty_clear()
 {
     tty_io_clear();
 
-    memset(tty_scr,      '\0', lines * columns * sizeof(unsigned char));
-    memset(tty_atr,      '\0', lines * columns * sizeof(unsigned char));
-    memset(tty_prev_scr, '\0', lines * columns * sizeof(unsigned char));
-    memset(tty_prev_atr, '\0', lines * columns * sizeof(unsigned char));
+    memset(tty_scr,      '\0', tty_lines*tty_columns * sizeof(unsigned char));
+    memset(tty_atr,      '\0', tty_lines*tty_columns * sizeof(unsigned char));
+    memset(tty_prev_scr, '\0', tty_lines*tty_columns * sizeof(unsigned char));
+    memset(tty_prev_atr, '\0', tty_lines*tty_columns * sizeof(unsigned char));
 
     tty_cursor_x = 0;
     tty_cursor_y = 0;
@@ -1079,9 +1103,9 @@ void
 tty_fill()
 {
     memset(tty_scr, ' ',
-	   lines * columns * sizeof(unsigned char));
+	   tty_lines * tty_columns * sizeof(unsigned char));
     memset(tty_atr, tty_current_attribute,
-	   lines * columns * sizeof(unsigned char));
+	   tty_lines * tty_columns * sizeof(unsigned char));
 
     tty_touch();
 }
@@ -1096,7 +1120,7 @@ tty_fill()
 void
 tty_touch()
 {
-    memset(tty_prev_scr, '\0', lines * columns * sizeof(unsigned char));
+    memset(tty_prev_scr, '\0', tty_lines*tty_columns * sizeof(unsigned char));
 }
 
 
@@ -1267,7 +1291,7 @@ tty_io_reversevid(status)
 
 
 /*
- * Set the brightness, foreground and background all together.
+ * Set the brightness, foreground and background altogether.
  */
 
 static void
@@ -1363,7 +1387,7 @@ tty_colors(brightness, foreground, background)
 void
 tty_beep()
 {
-    tty_putc(7);
+    tty_writec(7);
     tty_flush();
 }
 
@@ -1435,25 +1459,51 @@ tty_defaults()
 
 
 /*
- * Get a character from the terminal. For better performance on high loaded
- * systems, read all the data available.
+ * Extract the first key in the buffer.  If the 8th bit is set, return
+ * an ESC char first, then the key without the 8th bit.
+ *
+ * FIXME: It seems that the 8th bit can be used for parity as well.
+ * This case is not handled yet.  It should be configurable.
+ */
+
+static int
+tty_extract_key()
+{
+    int key = keybuf[keyindex];
+
+    if (key & 0x80)
+    {
+	keybuf[keyindex] &= 0x7F;
+	return key_ESC;
+    }
+
+    keyno--;
+    keyindex++;
+    return key;
+}
+
+
+/*
+ * Get a character from the terminal.  For better performance on high
+ * loaded systems, read all the data available.
  */
 
 int
 tty_getc()
 {
     if (keyno)
-	return keybuf[keyno--, keyindex++];
+	return tty_extract_key();
 
     /* No interrupt/quit character.  */
     tty_set_interrupt_char(-1);
 
-    for (keyindex = 0; (keyno = tty_read(keybuf, 1024)) < 0;);
+    for (keyindex = 0; (keyno = tty_read(keybuf, (size_t)1024)) < 0;)
+	;
 
     /* Restore ^G as the interrupt/quit character.  */
     tty_set_interrupt_char(key_INTERRUPT);
 
-    return keyno ? keybuf[keyno--, keyindex++] : -1;
+    return keyno ? tty_extract_key() : -1;
 }
 
 
@@ -1619,7 +1669,7 @@ tty_get_key(repeat_count)
 
     for (i = 0; i < MAX_KEY_LENGTH; i++)
     {
-	/* Handle brain-damaged key sequences correctly.  If  a  0  is
+	/* Handle brain-damaged key sequences correctly.  If a 0 is
 	   found, transform it in a 0xff.  I don't know how smart this
 	   is, but right know I don't feel like doing it otherwise.  */
 	if (c == 0)
@@ -1636,7 +1686,8 @@ tty_get_key(repeat_count)
 	if (key)
 	    break;
 
-	while ((c = tty_getc()) == -1);
+	while ((c = tty_getc()) == -1)
+	    ;
     }
 
     if (i == MAX_KEY_LENGTH)
@@ -1654,75 +1705,81 @@ tty_get_key(repeat_count)
 }
 
 
+#define columns_ok(x) ((x) >= MIN_TTY_COLUMNS && (x) <= MAX_TTY_COLUMNS)
+#define lines_ok(x)   ((x) >= MIN_TTY_LINES   && (x) <= MAX_TTY_LINES)
+
+
 /*
- * Retreive the screen size. Try ioctl(TIOCGWINSZ, ...) and, if it fails
- * or it is not supported, use the environment variables LINES & COLUMNS
- * (if possible). If both methods fail, use the default 80x24.
+ * Retrieve the screen size.  The default is to use the number of
+ * lines and columns specified in the environment.  If the values are
+ * not acceptable, default to the value returned by the winsz ioctl()
+ * call.  If these ones are not valid (or the ioctl() call to get the
+ * window size is not supported), just default to the values specified
+ * in the termcap description.  If this fails too, default to 80x24.
+ * Note that we try to get the best value separately for lines and for
+ * columns.
  */
 
 void
-tty_get_size(_columns, _lines)
-    int *_columns, *_lines;
+tty_get_size(columns, lines)
+    int *columns, *lines;
 {
     char *env;
+    int shell_lines = 0, shell_columns = 0;
+    int termcap_lines = 0, termcap_columns = 0;
 
 #ifdef HAVE_WINSZ
-    int result;
     struct winsize winsz;
     int winsz_lines = 0, winsz_columns = 0;
 #endif /* HAVE_WINSZ */
 
-    columns = lines = 0;
-
 #ifdef HAVE_WINSZ
-
 #if defined TIOCGSIZE && !defined TIOCGWINSZ
 #define TIOCGWINSZ TIOCGSIZE
 #endif
-
-    result = ioctl(TTY_OUTPUT, TIOCGWINSZ, &winsz);
-
-    if (result == 0)
+    if (ioctl(TTY_OUTPUT, TIOCGWINSZ, &winsz) == 0)
 	if (winsz.ws_col && winsz.ws_row)
 	{
 	    winsz_columns = winsz.ws_col;
 	    winsz_lines   = winsz.ws_row;
 	}
-
-    if (winsz_columns < MIN_TTY_COLUMNS || winsz_columns > MAX_TTY_COLUMNS)
-	winsz_columns = 80;
-
-    if (winsz_lines < MIN_TTY_LINES || winsz_lines > MAX_TTY_LINES)
-	winsz_lines = 24;
-
 #endif /* HAVE_WINSZ */
 
     if ((env = getenv("COLUMNS")))
-	sscanf(env, "%d", &columns);
+	sscanf(env, "%d", &shell_columns);
 
     if ((env = getenv("LINES")))
-	sscanf(env, "%d", &lines);
+	sscanf(env, "%d", &shell_lines);
+
+    termcap_columns = TTY_COLUMNS;
+    termcap_lines   = TTY_LINES;
 
-    if (columns < MIN_TTY_COLUMNS || columns > MAX_TTY_COLUMNS)
-    {
 #ifdef HAVE_WINSZ
-	columns = winsz_columns;
-#else
-	columns = 80;
-#endif /* HAVE_WINSZ */
-    }
+    if (columns_ok(winsz_columns))
+	tty_columns = winsz_columns;
+    else
+#endif
+	if (columns_ok(shell_columns))
+	    tty_columns = shell_columns;
+	else if (columns_ok(termcap_columns))
+	    tty_columns = termcap_columns;
+	else
+	    tty_columns = 80;
 
-    if (lines < MIN_TTY_LINES || lines > MAX_TTY_LINES)
-    {
 #ifdef HAVE_WINSZ
-	lines = winsz_lines;
-#else
-	lines = 24;
-#endif /* HAVE_WINSZ */
-    }
+    if (lines_ok(winsz_lines))
+	tty_lines = winsz_lines;
+    else
+#endif
+	if (lines_ok(shell_lines))
+	    tty_lines = shell_lines;
+	else if (lines_ok(termcap_lines))
+	    tty_lines = termcap_lines;
+	else
+	    tty_lines = 24;
 
-    *_columns = columns;
-    *_lines   = lines;
+    *columns = tty_columns;
+    *lines   = tty_lines;
 }
 
 
@@ -1769,7 +1826,7 @@ vcs_io(buf, op)
     if (vcsfd != -1)
     {
       vcsa_label:
-	(*fn)(vcsfd, buf, 4 + lines * columns * 2);
+	(*fn)(vcsfd, buf, 4 + tty_lines * tty_columns * 2);
 	close(vcsfd);
 
 	if (op == VCS_WRITE)
@@ -1804,7 +1861,7 @@ vcs_io(buf, op)
     if (vcsfd != -1)
     {
       vcs_label:
-	(*fn)(vcsfd, buf, 4 + lines * columns);
+	(*fn)(vcsfd, buf, 4 + tty_lines * tty_columns);
 	close(vcsfd);
 
 	/* Unfortunately, the b/w devices do not provide the 4 bytes header
@@ -1812,7 +1869,7 @@ vcs_io(buf, op)
 
 	if (op == VCS_WRITE)
 	{
-	    tty_io_goto(lines - 1, 0);
+	    tty_io_goto(tty_lines - 1, 0);
 	    tty_flush();
 	}
 	return 1;
@@ -1871,7 +1928,8 @@ tty_put_screen(buf)
 	    if (vcs_io(buf, VCS_WRITE) == 0)
 		tty_clear();
 	    else
-		memset(tty_scr, '\0', lines * columns * sizeof(unsigned char));
+		memset(tty_scr, '\0',
+		       tty_lines * tty_columns * sizeof(unsigned char));
 	}
 	else
 	    tty_clear();
@@ -1903,7 +1961,8 @@ tty_get_color_index(colorname)
 
 
 /*
- * Return the capability of a given termcap symbol.
+ * Get the capability of a given termcap symbol.  Return NULL if there
+ * is no capability for it.
  */
 
 char *
@@ -1912,7 +1971,7 @@ tty_get_symbol_key_seq(symbol)
 {
     int i;
 
-    for (i = TTY_FIRST_SYMBOL_KEY; i < TTY_USED_CAPABILITIES; i++)
+    for (i = TTY_FIRST_SYMBOL_KEY; i < TTY_CAPABILITIES_USED; i++)
 	if (strcmp(tty_capability[i].symbol, symbol) == 0)
 	    return tty_capability[i].string;
 
@@ -1960,7 +2019,11 @@ tty_get_capabilities()
 	termtype = vt100;
     }
 
+#ifdef HAVE_LINUX
+    err = tgetent(NULL, termtype);
+#else
     err = tgetent(term_buf, termtype);
+#endif
 
     if (err == -1)
     {
@@ -1978,6 +2041,7 @@ tty_get_capabilities()
 		program, termtype, term_database);
 	exit(1);
     }
+
     tty_type = xstrdup(termtype);
 
     capability_buf = xmalloc(2048);
@@ -2019,6 +2083,12 @@ tty_get_capabilities()
 	TTY_ATTRIBUTES_OFF = TTY_REVERSE_ON = TTY_BRIGHT_ON = NULL;
 
 
+    /* Try to figure out the number of lines and columns as specified
+       in the termcap description.  */
+    TTY_COLUMNS = tgetnum(TTY_COLUMNS_NAME);
+    TTY_LINES   = tgetnum(TTY_LINES_NAME);
+
+
     TTY_CURSOR_OFF = tgetstr(TTY_CURSOR_OFF_NAME, &capability_buf);
     TTY_CURSOR_ON  = tgetstr(TTY_CURSOR_ON_NAME,  &capability_buf);
 
@@ -2028,11 +2098,11 @@ tty_get_capabilities()
     TTY_CLEAR_SCREEN = tgetstr(TTY_CLEAR_SCREEN_NAME, &capability_buf);
     TTY_CURSOR_MOVE  = tgetstr(TTY_CURSOR_MOVE_NAME,  &capability_buf);
 
-    for (i = TTY_FIRST_SYMBOL_KEY; i < TTY_USED_CAPABILITIES; i++)
+    for (i = TTY_FIRST_SYMBOL_KEY; i < TTY_CAPABILITIES_USED; i++)
 	tty_capability[i].string = tgetstr(tty_capability[i].name,
-						 &capability_buf);
+					   &capability_buf);
 
-    for (i = 0; i < TTY_USED_CAPABILITIES; i++)
+    for (i = 0; i < TTY_CAPABILITIES_USED; i++)
 	if (tty_capability[i].string == NULL)
 	    if (tty_capability[i].required)
 	    {
@@ -2041,6 +2111,12 @@ tty_get_capabilities()
 			"%s: can't find the '%s' terminal capability.\n",
 			program, tty_capability[i].name);
 	    }
+	    else
+	    {
+		/* If this capability does not exist but its presence
+		   is not mandatory then make it be the null string.  */
+		tty_capability[i].string = "";
+	    }
 
     if (term_errors)
     {
