@@ -1,6 +1,6 @@
-/* window.c -- a *very* simple window management. */
+/* window.c -- A *very* simple window management.  */
 
-/* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1993, 1994, 1995, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,6 +31,12 @@
 #include "ansi_stdlib.h"
 #endif /* !HAVE_STDLIB_H */
 
+#include <sys/types.h>
+
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
 #include "window.h"
 #include "xmalloc.h"
 #include "tty.h"
