@@ -1,6 +1,6 @@
-/* window.h -- the data structures and function prototypes used by window.c. */
+/* window.h -- Data structures and function prototypes used by window.c.  */
 
-/* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1993, 1994, 1995, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,6 +25,10 @@
 
 #include <sys/types.h>
 
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
 #include "stdc.h"
 
 
@@ -35,14 +39,13 @@ typedef struct
 } window_t;
 
 
-extern window_t *window_init	PROTO ((size_t, size_t, size_t, size_t));
-extern void window_end		PROTO ((window_t *));
-extern void window_resize	PROTO ((window_t *, size_t, size_t,
-					size_t, size_t));
-extern size_t window_puts	PROTO ((char *, size_t));
-extern size_t window_putc	PROTO ((char));
-extern void window_goto		PROTO ((window_t *, size_t, size_t));
-extern void window_update	PROTO (());
+extern window_t *window_init PROTO ((size_t, size_t, size_t, size_t));
+extern void window_end PROTO ((window_t *));
+extern void window_resize PROTO ((window_t *, size_t, size_t, size_t, size_t));
+extern size_t window_puts PROTO ((char *, size_t));
+extern size_t window_putc PROTO ((char));
+extern void window_goto PROTO ((window_t *, size_t, size_t));
+extern void window_update PROTO (());
 
 
 #endif  /* _GIT_WINDOW_H */
