@@ -1,5 +1,5 @@
-/* xid.c -- cached versions of the getpwuid/getgrgid functions. These
-   versions return only the user/group name, not the entire structure. */
+/* xid.c -- Cached versions of the getpwuid/getgrgid functions.  These
+   versions return only the user/group name, not the entire structure.  */
 
 /* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
 
@@ -55,9 +55,11 @@ xuid_t *uid_buckets[UID_BUCKETS];
 xgid_t *gid_buckets[GID_BUCKETS];
 
 
-/* Search uid in the hash table. If it is there, return a pointer to the
-   coresponding string. If it is not, find the /etc/passwd entry and insert
-   it in the hash table.  */
+/*
+ * Search uid in the hash table. If it is there, return a pointer to the
+ * coresponding string. If it is not, find the /etc/passwd entry and insert
+ * it in the hash table.
+ */
 
 char *
 xgetpwuid(uid)
@@ -123,7 +125,10 @@ xgetgrgid(gid)
 }
 
 
-/* Initialize the hash tables. */
+/*
+ * Initialize the hash tables.
+ */
+
 void
 xid_init()
 {
