@@ -1,6 +1,6 @@
-/* xio.c -- safe versions of the system io primitives. It also includes a
+/* xio.c -- Safe versions of the system io primitives.  It also includes a
    new version of the readlink system call that computes the number of
-   characters required to hold the entire link. */
+   characters required to hold the entire link.  */
 
 /* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
 
@@ -27,6 +27,10 @@
 
 #include <sys/types.h>
 
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
@@ -53,6 +57,7 @@ extern int errno;
 
 
 #include "xmalloc.h"
+#include "xstring.h"
 #include "xio.h"
 
 
@@ -110,6 +115,67 @@ xwrite(fd, buf, count)
 }
 
 
+#ifndef HAVE_RENAME
+
+/*
+ * This is a rename() function for the systems that don't have one.
+ * Stolen from glibc-1.08.1.
+ */
+
+int
+rename(old, new)
+    const char *old;
+    const char *new;
+{
+    int save = errno;
+
+    if (link(old, new) < 0)
+    {
+	if (errno == EEXIST)
+	{
+	    errno = save;
+
+	    /* Race condition, required for 1003.1 conformance.  */
+	    if (unlink(new) < 0 || link(old, new) < 0)
+		return -1;
+	}
+	else
+	    return -1;
+    }
+
+    if (unlink(old) < 0)
+    {
+	save = errno;
+
+	if (unlink(new) == 0)
+	    errno = save;
+
+	return -1;
+    }
+
+    return 0;
+}
+#endif /* HAVE_RENAME */
+
+
+#ifndef HAVE_READLINK
+
+/*
+ * readlink() stub.  Just to make things compile.
+ */
+
+int
+readlink(path, buf, bufsize)
+    const char *path;
+    char *buf;
+    size_t bufsize;
+{
+    /* The underlying system doesn't have the readlink() system call.  */
+    return -1;
+}
+#endif /* HAVE_READLINK */
+
+
 int
 __xreadlink(path, buf, size)
     const char *path;
@@ -149,7 +215,7 @@ xreadlink(filename)
     for (;;)
     {
 	char *buffer = xmalloc(size);
-	int  nchars  = __xreadlink(filename, buffer, size);
+	int nchars = __xreadlink(filename, buffer, (size_t)size);
 
 	if (nchars < size)
 	{
@@ -215,28 +281,33 @@ xlstat(filename, buf)
 char *
 xgetcwd()
 {
-    size_t size;
     char *result;
     char *cwd;
 
     errno = 0;
 
 #ifdef HAVE_GETCWD
-    cwd = xmalloc(size = 64);
-
-    while ((result = getcwd(cwd, size)) == NULL && errno == ERANGE)
     {
-	cwd = xrealloc(cwd, size += 64);
-	errno = 0;
+	size_t size;
+
+	cwd = xmalloc(size = 64);
+
+	while ((result = getcwd(cwd, size)) == NULL && errno == ERANGE)
+	{
+	    cwd = xrealloc(cwd, size += 64);
+	    errno = 0;
+	}
     }
 #else
-    /* No getcwd() -> getwd(): BSD style... bleah!  */
+    {
+	/* No getcwd() -> getwd(): BSD style... bleah!  */
 
-    cwd    = xmalloc(MAXPATHSIZE + 2);
-    result = getwd(cwd);
+	cwd    = xmalloc(MAXPATHSIZE + 2);
+	result = getwd(cwd);
 
-    if (result)
-	cwd = xrealloc(cwd, strlen(cwd) + 1);
+	if (result)
+	    cwd = xrealloc(cwd, strlen(cwd) + 1);
+    }
 #endif  /* HAVE_GETCWD */
 
     if (result == NULL)
@@ -249,3 +320,18 @@ xgetcwd()
 
     return cwd;
 }
+
+
+char *
+xbasename(name)
+    char *name;
+{
+    char *base;
+    size_t len = strlen(name);
+
+    if (name[len - 1] == '/')
+	name[len - 1] = '\0';
+
+    base = strrchr(name, '/');
+    return base ? base + 1 : name;
+}
