@@ -1,4 +1,4 @@
-/* xio.h -- function prototypes used in xio.c. */
+/* xio.h -- Function prototypes used in xio.c.  */
 
 /* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
 
@@ -25,20 +25,33 @@
 
 #include <sys/types.h>
 
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
 #include "stat.h"
 #include "stdc.h"
 
 
-extern int xread	PROTO ((int, char *, size_t));
-extern int xwrite	PROTO ((int, const char *, size_t));
+extern int xread PROTO ((int, char *, size_t));
+extern int xwrite PROTO ((int, const char *, size_t));
 
-extern int xreadlink	PROTO ((const char *));
+#ifndef HAVE_RENAME
+extern int rename PROTO ((const char *, const char *));
+#endif /* HAVE_RENAME */
 
-extern int xfstat	PROTO ((int, struct stat *));
-extern int xstat	PROTO ((const char *, struct stat *));
-extern int xlstat	PROTO ((const char *, struct stat *));
+#ifndef HAVE_READLINK
+extern int readlink PROTO ((const char *, char *, size_t));
+#endif /* HAVE_READLINK */
 
-extern char *xgetcwd	PROTO (());
+extern int xreadlink PROTO ((const char *));
+
+extern int xfstat PROTO ((int, struct stat *));
+extern int xstat PROTO ((const char *, struct stat *));
+extern int xlstat PROTO ((const char *, struct stat *));
+
+extern char *xgetcwd PROTO (());
+extern char *xbasename PROTO ((char *));
 
 
 #endif /* _GIT_XIO_H */
