@@ -1,4 +1,4 @@
-/* xmalloc.h -- prototypes for the functions in xmalloc.c. */
+/* xmalloc.h -- Prototypes for the functions in xmalloc.c.  */
 
 /* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
 
@@ -25,6 +25,10 @@
 
 #include <sys/types.h>
 
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #else /* !HAVE_STDLIB_H */
@@ -34,9 +38,10 @@
 #include "stdc.h"
 
 
-extern char *xmalloc  PROTO ((size_t));
-extern char *xcalloc  PROTO ((size_t, size_t));
+extern char *xmalloc PROTO ((size_t));
+extern char *xcalloc PROTO ((size_t, size_t));
 extern char *xrealloc PROTO ((void *, size_t));
-extern void  xfree    PROTO ((void *));
+extern void  xfree PROTO ((void *));
+
 
 #endif  /* _GIT_XMALLOC_H */
